"""file display module"""
import os
import PyQt5.QtWidgets as qt
import PyQt5.QtCore as qtc
from abf_explorer.abf_logging import make_logger

# TypeError: on_dir_changed() missing 1 required positional argument:
# 'current_dicts' when calling choose dir button. Why?
# https://doc.qt.io/qtforpython/overviews/qtwidgets-tutorials-addressbook-part1-example.html#part-1-designing-the-user-interface
# TODO! signal changes. Print new file on change. This may need to be set in
# the main controller class? Alternatively, could add a listener to a VAR here
# for the main class to watch and take action.

logger = make_logger(__name__)


class FileDisplay(qt.QWidget):
    """controls display and file handling for file selection"""

    # signals
    dirchanged = qtc.pyqtSignal(tuple)
    selectionchanged = qtc.pyqtSignal(str)

    def __init__(self, parent, command_line_dir=""):
        super().__init__(parent=parent)
        # VARS
        self._var_working_dir = os.path.expanduser(
            "~"
        )  # start home, replace with prev dir after selection

        # Interaction and signal #
        self.button_select_abf = qt.QPushButton("Choose folder")
        self.button_select_abf.clicked.connect(self.choose_directory_button_activated)

        # Listbox and signal #
        self.listbox_file_list = qt.QListWidget()
        self.listbox_file_list.currentItemChanged.connect(self.on_selection_changed)
        # layout
        self.layout = qt.QVBoxLayout()
        self.layout.addWidget(self.button_select_abf)
        self.layout.addWidget(self.listbox_file_list)
        self.setLayout(self.layout)

        # Actions

        if command_line_dir:
            self.input_dir(command_line_dir)

    def input_dir(self, path):
        logger.debug(f"called with: {path}")
        current_dicts = self._filter_and_make_dict(path)
        current_selection = self._populate_listbox_file_list(current_dicts)
        self.on_dir_changed(current_selection, current_dicts)
        return (current_selection, current_dicts)

    def on_dir_changed(self, current_selection, current_dicts):
        logger.debug(f"emitting tuple {(current_selection, current_dicts)}")
        self.dirchanged.emit((current_selection, current_dicts))

    def on_selection_changed(self, *args):
        logger.debug("selection changed")
        current = args[0]
        if not current:
            logger.debug(f"problem! selections passed: {args}")
            return
        current = current.text()
        self.selectionchanged.emit(current)
        logger.debug(f"Selection changed, emitting: {current}")
        return

    def choose_directory_button_activated(self):
        """sets file listbox and returns current selection and shortname:full-path dict.
        activated when button pushed. Checks for valid files (abf only now),
        sets the listbox with the file paths
        :param command_line_dir: a string passed from --startup-dir or -d upon app
        startup, defaults to None.
        :return: a tuple of current_selection and a dictionary where keys are the base
        file name and vals are the full paths to the files.
        """
        selected_dir = qt.QFileDialog().getExistingDirectory(
            self, "Select ABF Directory", self._var_working_dir
        )
        logger.debug(f"selected dir is {selected_dir}")
        # case when button is cancelled.
        if selected_dir is None:
            logger.debug("button action likely cancelled by user.")
            logger.warning("button action likely cancelled by user.")
            self.on_dir_changed(None, None)
            return
        self.on_dir_changed(self.input_dir(selected_dir))
        return

    def _filter_and_make_dict(self, directory):
        if not directory:
            logger.warning(f"Invalid directory: {directory}")
            return {"No ABFs found": "ABF directory error"}
        if not os.path.exists(directory):
            logger.warning(f"Directory does not exist: {directory}")
            return {"No ABFs found": "ABF directory error"}
        logger.debug(f"setting working dir to {directory}")
        self._var_working_dir = directory
        abfs = [abf for abf in os.listdir(directory) if abf.endswith("abf")]
        if len(abfs) < 1:
            logger.warning(f"No ABFs found in: {directory}")
            return {"Nothing here...": "Bad directory"}
        current_dicts = {f: os.path.join(directory, f) for f in abfs}
        return current_dicts

    def _populate_listbox_file_list(self, selected_abf_files_dict):
        self.listbox_file_list.clear()
        sorted_keys = sorted(selected_abf_files_dict.keys())
        for n, f in enumerate(sorted_keys):
            self.listbox_file_list.insertItem(n, f)
        default_selection = self.listbox_file_list.item(0)
        self.listbox_file_list.setCurrentItem(default_selection)
        return sorted_keys[0]



"""This module displays data for file metadata"""
import PyQt5.QtWidgets as qt
import PyQt5.QtCore as qtc
from abf_explorer.abf_logging import make_logger


logger = make_logger(__name__)


class FileInfoPlotControls(qt.QWidget):
    """class for display of info from selected file and controlling plots."""

    sendselections = qtc.pyqtSignal(tuple)
    clearplot = qtc.pyqtSignal(bool)

    def __init__(self, parent=None):
        super().__init__(parent)
        # VARS
        # layout control widgets
        self.file_info_widget = qt.QWidget()
        self.plot_controls_widget = qt.QWidget()

        # buttons and displays
        self.button_plot_controls_plot = qt.QPushButton("plot")
        self.button_plot_controls_plot.setToolTip(
            "add selected data to the plot ('Tab')"
        )
        self.button_plot_controls_plot.clicked.connect(
            self.get_sweep_and_channel_plotting_opts
        )

        self.button_plot_controls_clear_plot = qt.QPushButton("clear plot")
        self.button_plot_controls_clear_plot.setToolTip("clear plot ('c')")
        self.button_plot_controls_clear_plot.clicked.connect(self.emit_clear_plot)

        self.combobox_plot_controls_sweep_list = qt.QComboBox()
        self.combobox_plot_controls_channel_list = qt.QComboBox()

        self.label_file_info_file_name = qt.QLabel("File name:")
        self.label_file_info_file_name_val = qt.QLabel("")
        self.label_file_info_protocol = qt.QLabel("Protocol:")
        self.label_file_info_protocol_val = qt.QLabel("")
        self.label_file_info_sampling_frequency = qt.QLabel("Sampling Freq. (kHz):")
        self.label_file_info_sampling_frequency_val = qt.QLabel("")

        # layouts
        self.main_layout = qt.QGridLayout()
        self.file_info_layout_form = qt.QFormLayout()
        self.plot_controls_layout = qt.QGridLayout()

        self.file_info_layout_form.addRow(
            self.label_file_info_file_name, self.label_file_info_file_name_val
        )
        self.file_info_layout_form.addRow(
            self.label_file_info_protocol, self.label_file_info_protocol_val
        )
        self.file_info_layout_form.addRow(
            self.label_file_info_sampling_frequency,
            self.label_file_info_sampling_frequency_val,
        )
        self.plot_controls_layout.addWidget(
            self.combobox_plot_controls_sweep_list, 0, 0
        )
        self.plot_controls_layout.addWidget(
            self.combobox_plot_controls_channel_list, 1, 0
        )
        self.plot_controls_layout.addWidget(self.button_plot_controls_plot, 0, 1)
        self.plot_controls_layout.addWidget(self.button_plot_controls_clear_plot, 1, 1)

        self.main_layout.addWidget(
            self.file_info_widget, 0, 0, alignment=qtc.Qt.AlignLeft
        )
        self.main_layout.addWidget(self.plot_controls_widget, 0, 1)

        self.file_info_widget.setLayout(self.file_info_layout_form)
        self.plot_controls_widget.setLayout(self.plot_controls_layout)
        self.setLayout(self.main_layout)

    def update_metadata_vals(self, file_metadata_dict):
        self.label_file_info_file_name_val.setText(
            file_metadata_dict.get("short_filename")
        )
        self.label_file_info_protocol_val.setText(file_metadata_dict.get("protocol"))
        self.label_file_info_sampling_frequency_val.setText(
            file_metadata_dict.get("sampling_frequency_khz")
        )
        self.combobox_plot_controls_sweep_list.clear()
        self.combobox_plot_controls_sweep_list.addItems(
            [
                "sweep " + str(sweep)
                for sweep in range(file_metadata_dict.get("n_sweeps", 0))
            ]
        )
        self.combobox_plot_controls_channel_list.clear()
        self.combobox_plot_controls_channel_list.addItems(
            [
                "channel " + str(sweep)
                for sweep in range(file_metadata_dict.get("n_channels", 0))
            ]
        )
        logger.debug("updated metadata")

    def get_sweep_and_channel_plotting_opts(self):
        sweep_ind = self.combobox_plot_controls_sweep_list.currentIndex()
        channel_ind = self.combobox_plot_controls_channel_list.currentIndex()
        self.sendselections.emit((sweep_ind, channel_ind))
        logger.debug(f"emitting tuple sweep: {sweep_ind}, channel: {channel_ind}")

    def emit_clear_plot(self):
        logger.debug("emitting clear plot")
        self.clearplot.emit(True)



"""main GUI interface class"""
# pyqt and graph testing
# https://doc.qt.io/qtforpython/PySide2/QtWidgets/QWidget.html
# https://doc.qt.io/qt-5/layout.html
# https://pythonbasics.org/pyqt-grid/
# examples python -m pyqtgraph.examples

import PyQt5.QtWidgets as qt
import PyQt5.QtCore as qtc
from PyQt5 import QtGui

import abf_explorer
from abf_explorer.abf_logging import make_logger
from abf_explorer.filedisplay import FileDisplay
from abf_explorer.fileinfoplotcontrols import FileInfoPlotControls
from abf_explorer.plotting import PlotWidget
from abf_explorer import plotutils
from abf_explorer import abf


logger = make_logger(__name__)


class ABFExplorer(qt.QMainWindow):
    """main class contains all widgets and coordinates all actions"""

    # signals
    metadatachanged = qtc.pyqtSignal(dict)
    sendplotdata = qtc.pyqtSignal(dict)

    def __init__(self, startup_dir=""):
        super().__init__()
        self.startup_dir = startup_dir
        self.central_widget = qt.QWidget()
        self.setCentralWidget(self.central_widget)
        self.setWindowTitle(f"ABF Explorer v{abf_explorer.__version__}")
        # TODO menu bar
        # vars
        self.var_current_selection_short_name = ""
        self.var_selected_abf_files_dict = {}

        self.var_current_metadata_dict = {}
        self.var_currently_plotted_data = {}
        self.var_x_units_plotted = ""
        self.var_y_units_plotted = ""

        # make widgets and connect signals
        self._init_plot_widget()
        self._init_file_explorer()
        self._init_file_info_plot_controls()

        # main widget layout and geometry
        self.main_layout = qt.QGridLayout()
        self.main_layout.setColumnStretch(0, 1)
        self.main_layout.setColumnStretch(1, 5)
        self.main_layout.setColumnMinimumWidth(0, 15)
        self.main_layout.setColumnMinimumWidth(1, 400)

        self.main_layout.addWidget(self.file_explorer_widget, 0, 0, 1, 1)
        self.main_layout.addWidget(self.plot_widget, 0, 1)
        self.main_layout.addWidget(self.file_info_plot_controls_widget, 1, 0, 1, 2)
        self.main_layout.addWidget(self.file_info_plot_controls_widget, 1, 1)
        self.central_widget.setLayout(self.main_layout)

        # keyboard shortcuts

        # geometry and run
        self.setGeometry(50, 50, 900, 600)
        self.show()

    def _init_file_explorer(self):
        logger.debug("Initializing FileDisplay")
        self.file_explorer_widget = FileDisplay(parent=self.central_widget)
        self.file_explorer_widget.dirchanged.connect(
            self.update_current_directory_and_selection
        )

        if self.startup_dir:
            logger.debug(f"startup dir passed: {self.startup_dir}")
            startup_stuff = self.file_explorer_widget.input_dir(self.startup_dir)
            self.update_current_directory_and_selection(startup_stuff)

        self.file_explorer_widget.selectionchanged.connect(
            self.update_current_selection_and_metadata
        )
        return

    def _init_file_info_plot_controls(self):
        logger.debug("Initializing FileInfoPlotControls")
        self.file_info_plot_controls_widget = FileInfoPlotControls(
            parent=self.central_widget
        )

        self.metadatachanged.connect(
            self.file_info_plot_controls_widget.update_metadata_vals
        )
        self.file_info_plot_controls_widget.sendselections.connect(self.send_to_plot)
        self.file_info_plot_controls_widget.clearplot.connect(self.clear_plot)
        self.shortcut_update_plot = qt.QShortcut(
            QtGui.QKeySequence("Tab"), self.central_widget
        )
        self.shortcut_update_plot.activated.connect(
            self.file_info_plot_controls_widget.get_sweep_and_channel_plotting_opts
        )

        self.shortcut_clear_plot = qt.QShortcut(
            QtGui.QKeySequence("c"), self.central_widget
        )
        self.shortcut_clear_plot.activated.connect(
            self.file_info_plot_controls_widget.emit_clear_plot
        )
        self.broadcast_metadata()
        return

    def _init_plot_widget(self):
        logger.debug("Initializing PlotWidget")
        self.plot_widget = PlotWidget(parent=self.central_widget)
        self.sendplotdata.connect(self.plot_widget.update_plot)

    def send_to_plot(self, sweep_and_channel):
        sweep, channel = sweep_and_channel
        logger.debug(f"updating metadata sweep: {sweep} channel: {channel}")
        plot_opts = plotutils.io_gather_plot_data(
            self.var_current_metadata_dict, sweep, channel,
        )

        status, fmt_plot_opts = plotutils.check_fmt_opts(
            self.var_currently_plotted_data, plot_opts, self.var_y_units_plotted,
        )
        if status == "unit_error":
            logger.warning(
                f"Unit mismatch, can't plot '{self.var_y_units_plotted}' on the same axis"
            )
            return
        if status == "unchanged":
            logger.debug("unchanged, continuing")
            return
        if status == "updated":
            logger.debug("updating plot")
            self.var_y_units_plotted = plot_opts["y_units"]
            self.var_currently_plotted_data = fmt_plot_opts
            self.sendplotdata.emit(plot_opts)
            logger.debug(f"emitting {plot_opts}")
            return
        else:
            logger.warning("no paths taken.")

    def broadcast_metadata(self):
        logger.debug("sending metadata")
        self.metadatachanged.emit(self.var_current_metadata_dict.copy())
        return

    def update_current_directory_and_selection(self, item: tuple):
        current, all_dict = item
        logger.debug(f"setting with selection {current}")
        self.var_current_selection_short_name = current
        logger.debug("setting abf_dict")
        self.var_selected_abf_files_dict = all_dict
        logger.debug("updating metadata")
        item_path = self.var_selected_abf_files_dict.get(
            self.var_current_selection_short_name
        )
        self.current_selected_object = abf.Abf(item_path)
        self.var_current_metadata_dict = self.current_selected_object.return_metadata()
        logger.debug(f"metadata is: {self.var_current_metadata_dict}")
        return

    def update_current_selection_and_metadata(self, newselection):
        logger.debug(f"updating current selection to {newselection} and metadata")
        self.var_current_selection_short_name = newselection
        item_path = self.var_selected_abf_files_dict.get(
            self.var_current_selection_short_name
        )
        self.current_selected_object = abf.Abf(item_path)
        self.var_current_metadata_dict = self.current_selected_object.return_metadata()
        self.broadcast_metadata()
        return

    def clear_plot(self, *args):
        """clears plot and currently_plotted_items"""
        self.plot_widget.clear_plot()
        self.var_currently_plotted_data = {}
        self.var_y_units_plotted = ""

    def set_linear_selection_region(self, bounds):
        logger.debug("setting linear selection region")
        self.plot_widget.make_linear_region(bounds)

    def get_linear_region_bounds(self):
        logger.debug("returning bounds")
        return list(self.plot_widget.var_linear_region_x_bounds).copy()

    def reset_linear_region(self, bounds):
        logger.debug(f"resetting linear region to {bounds}")
        self.plot_widget.reset_linear_region(bounds)



# command line options and parsing
import argparse

parser = argparse.ArgumentParser()
parser.add_argument(
    "-d",
    "--startup-dir",
    dest="startup_dir",
    help="Load ABFs from this directory upon startup.",
)



import os
import numpy as np
import pyabf
from abf_explorer.abf_logging import make_logger

logger = make_logger(__name__)


class Abf:
    def __init__(self, abf_path):
        logger.debug(f"passed {abf_path}")

        self.var_abf_path = ""
        self._var_abf = None
        self.error = None
        self._var_plot_data = {
            "short_filename": "",
            "full_path": "",
            "sampling_frequency_khz": "",
            "protocol": "",
            "n_sweeps": 0,
            "n_channels": 0,
            "target_sweep": None,
            "mean_sweeps": False,
            "filtered_sweeps": False,
            "error": None,
        }
        self._validate_path(abf_path)

    def _validate_path(self, path):
        """verify path exists and is an ABF file"""

        if not path.endswith(".abf"):
            err_str = f"path: {path} does not end in `.abf`"
            logger.warning(err_str)
            self.error = err_str
            self.var_abf_path = ""
            return
        if not os.path.exists(path):
            err_str = f"path: {path} does not exist"
            logger.warning(err_str)
            self.error = err_str
            self.var_abf_path = ""
            return
        self.error = None
        self.var_abf_path = path

    def return_metadata(self):
        metadata = self._var_plot_data.copy()
        metadata["short_filename"] = os.path.split(self.var_abf_path)[-1].replace(
            ".abf", ""
        )
        metadata["full_path"] = self.var_abf_path
        if self.error:
            return self._var_plot_data.copy()
        try:
            self._var_abf = pyabf.ABF(self.var_abf_path)
        except Exception as e:
            err_str = f"cannot read file : {metadata['full_path']}. likely pyabf error. exception is : {e}"
            metadata["error"] = err_str
            logger.warning(err_str)
            return metadata
        metadata["sampling_frequency_khz"] = str(self._var_abf.dataRate / 1000)
        metadata["protocol"] = str(self._var_abf.protocol)
        metadata["n_sweeps"] = self._var_abf.sweepCount
        metadata["n_channels"] = self._var_abf.channelCount
        logger.debug(f"{metadata}")
        return metadata

    def _get_sweep_and_channel_data(self, channel, sweep):
        self._var_abf.setSweep(channel=channel, sweep=sweep)
        return (
            self._var_abf.sweepX,
            self._var_abf.sweepY,
            self._var_abf.sweepUnitsX,
            self._var_abf.sweepUnitsY,
        )

    def send_plot_data(self, channel, sweep):
        metadata = self.return_metadata()
        metadata["channel"] = channel
        metadata["sweep"] = sweep
        x, y, x_units, y_units = self._get_sweep_and_channel_data(channel, sweep)
        metadata["x"] = x
        metadata["y"] = y
        metadata["x_units"] = x_units
        metadata["y_units"] = y_units
        metadata["name"] = f"{metadata['short_filename']} sweep-{sweep} ch-{channel}"



"""plotting widget"""
from itertools import cycle
import pyqtgraph as pg
from abf_explorer.abf_logging import make_logger

pg.setConfigOption("background", "w")
pg.setConfigOption("foreground", "k")

# each thing plotted needs to be a distinct plotdataitem added to the plotitem
# https://pyqtgraph.readthedocs.io/en/latest/graphicsItems/plotdataitem.html
# https://pyqtgraph.readthedocs.io/en/latest/graphicsItems/plotitem.html

logger = make_logger(__name__)


class PlotWidget(pg.GraphicsWindow):
    def __init__(self, parent):
        super().__init__(parent=parent)
        self.color_list = [
            "#e41a1cff",
            "#377eb8ff",
            "#4daf4aff",
            "#984ea3ff",
            "#ff7f00ff",
            "#ffff33ff",
            "#a65628ff",
            "#f781bfff",
            "#999999ff",
        ]
        self.linear_region = None
        self.data = None
        self.color_cycler = cycle(self.color_list.copy())
        self.pen_width = 2
        self.set_main_canvas()
        self.var_linear_region_x_bounds = [None, None]

    def print_clicked(self, *args):
        logger.debug("Clicked")
        logger.debug(f"args: {[arg for arg in args]}")

    def _clear_legend(self):
        logger.debug("called")
        for sample, label in self.main_plot_widget.legend.items:
            self.main_plot_widget.legend.layout.removeItem(sample)
            self.main_plot_widget.legend.layout.removeItem(label)
        self.main_plot_widget.legend.items = []
        self.main_plot_widget.legend.updateSize()

    def _clear_plot_items(self):
        logger.debug("clearing items from plot")
        while self.main_plot_widget.items:
            self.main_plot_widget.removeItem(self.main_plot_widget.items[0])

    def set_main_canvas(self):
        self.main_plot_widget = self.addPlot(title="")
        self.main_plot_widget.addLegend()

    def update_plot(self, plotdict):
        self.main_plot_widget.legend.update()
        logger.debug(f"called. legend items are {self.main_plot_widget.legend.items}")
        self.data = pg.PlotDataItem(
            plotdict["x"],
            plotdict["y"],
            name=plotdict["name"],
            pen=pg.mkPen(self.color_cycler.__next__(), width=1,),
        )

        self.data.sigClicked.connect(self.print_clicked)
        self.main_plot_widget.addItem(self.data)
        self.main_plot_widget.setLabels(
            left=plotdict["y_units"], bottom=plotdict["x_units"]
        )

    def make_linear_region(self, bounds):
        self.linear_region = pg.LinearRegionItem(bounds, movable=True, swapMode="block")
        self.main_plot_widget.addItem(self.linear_region)
        self.linear_region.sigRegionChangeFinished.connect(
            self._emit_linear_region_x_bounds
        )
        self.var_linear_region_x_bounds = self._emit_linear_region_x_bounds()
        logger.debug(f"adding linear region with bounds {bounds}")

    def _emit_linear_region_x_bounds(self):
        logger.debug(
            f"linear region signal called, data bounds: {self.linear_region.dataBounds(axis=0)}"
        )
        self.var_linear_region_x_bounds = self.linear_region.dataBounds(axis=0)

    def reset_linear_region(self, bounds):
        logger.debug(f"resetting region to {bounds}")
        self.linear_region.setRegion(bounds)
        self.var_linear_region_x_bounds = self._emit_linear_region_x_bounds()

    def clear_plot(self, *args):
        logger.debug("called")
        self._clear_plot_items()
        logger.debug(f"main_plot items: {[i for i in self.main_plot_widget.items]}")
        self.linear_region = None
        self.data = None
        self.var_linear_region_x_bounds = [None, None]
        self._clear_legend()
        self.main_plot_widget.clearPlots()
        self.main_plot_widget.legend.update()
        self.main_plot_widget.setLabels(left="", bottom="")
        self.color_cycler = cycle(self.color_list.copy())



# utilities for interacting with pyabf
# would like a simpler way to handle IO. So when you select the ABF, we should store the ABF somewhere? or store the path to the ABF somewhere along with something like n_sweeps, n_channels, +metadata? this should be included in the metadata, along with the full path.
# Would this be simpler as a class? Verify the path once, then create the metadata map and autofill the opts_map with defaults?
import hashlib
import os
import numpy as np
import pyabf
from abf_explorer.abf_logging import make_logger

logger = make_logger(__name__)


PLOTDATA = {
    "short_filename": "",
    "full_path": "",
    "sampling_frequency_khz": "",
    "protocol": "",
    "n_sweeps": 0,
    "n_channels": 0,
    "target_sweep": None,
    "mean_sweeps": False,
    "filtered_sweeps": False,
    "error": None,
}


def io_get_metadata(abf_path):
    metadata = PLOTDATA.copy()
    if not abf_path:
        logger.debug("no abf path passed")
        logger.warning("no abf path passed")
        return metadata_error("", abf_path)

    if not abf_path.endswith(".abf"):
        logger.warning(f"string is not an abf: {abf_path}")
        return metadata_error(f"string is not an abf: {abf_path}")
    if not os.path.exists(abf_path):
        logger.warning(f"path does not exist: {abf_path}")
        return metadata_error(f"path does not exist: {abf_path}")
    if not os.path.isfile(abf_path):
        logger.warning(f"path passed is not a file: {abf_path}")
        return metadata_error(f"path passed is not a file: {abf_path}")
    try:
        abf = io_read_abf(abf_path, loadData=False)
        metadata["short_filename"] = abf.abfID
        metadata["full_path"] = abf_path
        metadata["sampling_frequency_khz"] = str(abf.dataRate / 1000)
        metadata["protocol"] = str(abf.protocol)
        metadata["n_sweeps"] = abf.sweepCount
        metadata["n_channels"] = abf.channelCount
        return metadata
    except AssertionError as e:
        logger.exception(e)
        return metadata_error(e, abf_path)


def io_read_abf(abf_path, loadData):
    try:
        logger.debug(f"reading ABF: {abf_path}")
        abf = pyabf.ABF(abf_path, loadData=loadData)
        return abf
    except Exception as e:
        logger.warning("exception likely thrown by pyABF")
        logger.exception(e)
        raise AssertionError("exception likely thrown by pyABF")


def mean_sweeps(abf, channel):
    l = []
    for sweep in abf.sweepList:
        abf.setSweep(sweep, channel=channel)
        l.append(abf.sweepY)
    l = np.asarray(l)
    return l.mean(axis=0)


def metadata_error(error, attempted_path=None):
    logger.warning(f"returning blank metadata")
    metadata = PLOTDATA.copy()
    metadata["full_path"] = attempted_path
    return metadata


def make_name(metadata_map):
    if metadata_map["mean_sweeps"] != False:
        return (
            f"{metadata_map['short_filename']} mean-sweeps ch-{metadata_map['channel']}"
        )
    if metadata_map["filtered_sweeps"] != False:
        raise (NotImplementedError)
    return f"{metadata_map['short_filename']} sweep-{metadata_map['sweep']} ch-{metadata_map['channel']}"


def _check_lfp_analysis(d):
    """if key LFP analysis is in the dict, return true"""
    if "_lfp_analysis" in d.keys():
        return True
    else:
        return False


def io_gather_plot_data(
    metadata_map, target_sweep, target_channel,
):
    """needs refactoring! each specialized transformation and hashing should have it's own fn.
    errors in those fns should raise"""
    mm = metadata_map.copy()
    try:
        abf = io_read_abf(mm["full_path"], loadData=True)
        hashed_id = hashlib.sha256(
            f"{mm['full_path']}-{target_sweep}-{target_channel}-{mm['mean_sweeps']}-{mm['filtered_sweeps']}".encode(
                "utf-8"
            )
        ).hexdigest()
        abf.setSweep(sweepNumber=target_sweep, channel=target_channel)
        mm["hashed_id"] = hashed_id
        mm["filtered_sweeps_data"] = None  # NOT IMPLEMENTED
        mm["channel"] = target_channel
        mm["sweep"] = target_sweep
        mm["x"] = abf.sweepX
        if mm["mean_sweeps"] == True:
            logger.debug(f"mean sweeps True: {mm['mean_sweeps']}")
            mm["y"] = mean_sweeps(abf, target_channel)
        if mm["mean_sweeps"] == False:
            logger.debug(f"mean sweeps True: {mm['mean_sweeps']}")
            mm["y"] = abf.sweepY
        if _check_lfp_analysis(mm):
            abf.setSweep(0, channel=1)
            mm["lfp_stim_data"] = abf.sweepY
        mm["x_units"] = abf.sweepUnitsX
        mm["y_units"] = abf.sweepUnitsY
        mm["name"] = make_name(mm)
        return mm
    except Exception as e:
        logger.exception(e)
        logger.warning(f"something went wrong. metadata_contents: {mm}")
        raise (AssertionError(f"Something went wrong.\n\nexception is {e}\n"))


def _check_y_units(new_y_units, existing_y_units):
    if existing_y_units == "":
        return
    if existing_y_units != 1:
        assert existing_y_units == new_y_units, "Unit mismatch"
        return


def check_fmt_opts(main_map, new_map, existing_y_units):
    mm = main_map.copy()
    nm = new_map.copy()
    hashed_id = nm.pop("hashed_id", None)
    try:
        _check_y_units(nm["y_units"], existing_y_units)
    except AssertionError as e:
        return ("unit_error", mm)
    if not hashed_id in mm.keys():
        # Dump extra vals to save space
        _ = nm.pop("x", None)
        _ = nm.pop("y", None)
        mm[hashed_id] = nm
        return ("updated", mm)

    print("[check_fmt_opts] Already plotted. Continuing")
    return ("unchanged", mm)



import re
from setuptools import setup

with open("README.md", "r") as instructions:
    README = instructions.read()

with open("abf_explorer/__init__.py", "r") as init:
    init_str = init.read()

version_matcher = re.compile(r"__version__ = (\".*\")")
version_str = version_matcher.search(init_str).group(1).strip('"')

setup(
    name="abf_explorer",
    author="Nick George",
    author_email="nicholas.george32@gmail.com",
    license="GPL v3",
    version=version_str,
    description="A PyQt GUI to quickly visualize ABF electrophysiology files",
    long_description=README,
    long_description_content_type="text/markdown",
    packages=["abf_explorer"],
    url="https://github.com/nkicg6/ABF_Explorer",
    python_requires=">=3.7",
    classifiers=[
        "License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)",
        "Programming Language :: Python :: 3.7",
        "Natural Language :: English",
        "Intended Audience :: Science/Research",
        "Topic :: Scientific/Engineering :: Medical Science Apps.",
        "Topic :: Scientific/Engineering :: Visualization",
    ],
    install_requires=[
        "numpy == 1.19.4",
        "PyQt5 == 5.15.1",
        "pyqtgraph == 0.11.0",
        "pyabf == 2.2.8",
    ],
    entry_points={"gui_scripts": ["abf_explorer = abf_explorer.__main__:main"]},
)



import os
import sys

import PyQt5.QtWidgets as qt
import PyQt5.QtCore as qtc
from abf_explorer.args import parser
from abf_explorer.gui import ABFExplorer
from abf_explorer.abf_logging import make_logger

logger = make_logger(__name__)

qt.QApplication.setAttribute(qtc.Qt.AA_EnableHighDpiScaling, True)


def main():
    """main for gui entrypoint"""
    cmd_args = parser.parse_args()
    app = qt.QApplication([])
    explorer = ABFExplorer(startup_dir=cmd_args.startup_dir)
    sys.exit(app.exec_())


if __name__ == "__main__":
    logger.debug("Starting ABF_Explorer")
    main()



import logging

logformatter = logging.Formatter(
    "-> [%(levelname)s] %(name)s::%(funcName)s() : %(message)s"
)
# unfortunately have to set the base logger format because pyABF sets it.
# That's why stuff is commented below
logging.basicConfig(
    level=logging.CRITICAL,
    format="-> [%(levelname)s] %(name)s::%(funcName)s() : %(message)s",
)


def make_logger(name):
    logger = logging.getLogger(name)
    logger.setLevel(logging.DEBUG)
    # console_log_handler = logging.StreamHandler()
    # console_log_handler.setLevel(logging.DEBUG)
    # console_log_handler.setFormatter(logformatter)
    # logger.addHandler(console_log_handler)
    curr_logs = [i for i in logging.Logger.manager.loggerDict.keys()]
    if "pyabf" in curr_logs:
        logging.getLogger("pyabf").setLevel(logging.CRITICAL)
    else:
        pass
    return logger
